@@ -177,16 +177,6 @@ def text(
     # with newline character's decimal reference.
     text = text.replace("\n\n", "&#10;&#10;")
 
-    # "===" and "---" sequences can seem like a header when aligned in
-    # a certain way. "---" sequence can also be interpreted as a
-    # thematic break. Escape them.
-    # TODO: This logic seems wrong? Only one char is needed for a heading
-    text = text.replace("===", r"\=\=\=")
-    text = text.replace("---", r"\-\-\-")
-    # "***" or "___" sequences can be interpreted as a thematic break. Escape them.
-    text = text.replace("***", r"\*\*\*")
-    text = text.replace("***", r"\_\_\_")
-
     # If the last character is a "!" and the token next up is a link, we
     # have to escape the "!" or else the link will be interpreted as image.
     if (
@@ -233,6 +223,10 @@ def _escape_asterisk_emphasis(text: str) -> str:
     Currently we escape all asterisks unless both previous and next
     character are Unicode whitespace.
     """
+    # Fast exit to improve performance
+    if "*" not in text:
+        return text
+
     escaped_text = ""
 
     text_length = len(text)
@@ -261,6 +255,10 @@ def _escape_underscore_emphasis(text: str) -> str:
         start or end of line, or Unicode punctuation
       - Both surrounding characters are Unicode whitespace
     """
+    # Fast exit to improve performance
+    if "_" not in text:
+        return text
+
     bad_neighbor_chars = (
         _codepoints.UNICODE_WHITESPACE
         | _codepoints.UNICODE_PUNCTUATION
